namespace CustomerData {
    using System;
    using System.ComponentModel;
    using System.Collections;
    using System.IO;

    public class Customer : IEditableObject, IDataErrorInfo {
       
        struct CustomerData {
            internal string id ;
            internal string title ;
            internal string firstName ;
            internal string lastName ;
            internal string address ;
            internal DateTime dateOfBirth;
        }
        
        private CustomersList parent;
        private CustomerData custData; 
        private CustomerData backupData; 
        private bool inTxn = false;

        /// <summary>
        ///     IEditableObject implementation
        /// </summary>
        void IEditableObject.BeginEdit() {
            Console.WriteLine("Start BeginEdit");
            if (!inTxn) {
                this.backupData = custData;
                inTxn = true;
                Console.WriteLine("BeginEdit - " + this.backupData.lastName);
            }
            Console.WriteLine("End BeginEdit");

        }

        void IEditableObject.CancelEdit() {
            Console.WriteLine("Start CancelEdit");
            if (inTxn) {
                this.custData = backupData;
                inTxn = false;
                Console.WriteLine("CancelEdit - " + this.custData.lastName);
            }
            Console.WriteLine("End CancelEdit");
        }

        void IEditableObject.EndEdit() {
            Console.WriteLine("Start EndEdit" + this.custData.id + this.custData.lastName);
            if (inTxn) {
                backupData = new CustomerData();
                inTxn = false;
                OnCustomerChanged();
                Console.WriteLine("Done EndEdit - " + this.custData.id + this.custData.lastName);
            }
            Console.WriteLine("End EndEdit");
        }


        /// <summary>
        ///     IDataErrorInfo implementation
        /// </summary>
        string IDataErrorInfo.Error { 
            get {
                IDataErrorInfo de = (IDataErrorInfo)this;
                if (!de["Address"].Equals("")) {
                    return "Your changes to this Customer are not valid";
                }
                return "" ;
            }
        }

        string IDataErrorInfo.this[ string propertyName ] { 
            get {
                if (propertyName.Equals("Address") && this.Address.Length < 2) {
                    return "The address is too short";
                }
                return "";
            }
        }




        public Customer(string ID) : base() {
            this.custData = new CustomerData();
            this.custData.id = ID;
            this.custData.title = "";
            this.custData.firstName = "";
            this.custData.lastName = "";
            this.custData.address = "";
            this.custData.dateOfBirth = DateTime.Now;
        }

        public string ID {
            get {
                return this.custData.id;
            }
        }
        
        public string FirstName {
            get {
                return this.custData.firstName;
            }
            set {
                this.custData.firstName = value;
                OnCustomerChanged();
            }
        }
        
        public string Title {
            get {
                return this.custData.title;
            }
            set {
                this.custData.title = value;
                OnCustomerChanged();
            }
        }
        
        public string LastName {
            get {
                return this.custData.lastName;
            }
            set {
                this.custData.lastName = value;
                OnCustomerChanged();
            }
        }

        public string Address {
            get {
                return this.custData.address;
            }
            set {
                this.custData.address = value;
                OnCustomerChanged();
            }
        }
        
        public DateTime DateOfBirth {
            get {
                return this.custData.dateOfBirth;
            }
            set {
                this.custData.dateOfBirth = value;
                OnCustomerChanged();
            }
        }
        
        public override string ToString() {
            StringWriter sb = new StringWriter();
            sb.Write(this.Title);
            sb.Write(" ");
            sb.Write(this.FirstName);
            sb.Write(" ");
            sb.Write(this.LastName);
            return sb.ToString();
        }

        internal CustomersList Parent {
            get {
                return parent;
            }
            set {
                parent = value ;
            }
        }
    
        private void OnCustomerChanged() {
            if (Parent != null) {
                Parent.CustomerChanged(this);
            }
        }
    
    }

}
                                             
