namespace CustomerData {
    using System;
    using System.ComponentModel;
    using System.Collections;

    public class CustomersList : CollectionsBase , IBindingList, IComponent  {
    
        private ListChangedEventArgs resetEvent = new ListChangedEventArgs(ListChangedType.Reset, -1);
        private ListChangedEventHandler onListChanged;
        private Component component = new Component();

        /// <summary>
        ///  Typed IList implementation
        /// </summary>
        public Customer this[int index] {
            get {
                return (Customer)(List[index]);
            }
            set {
                List[index] = value;
            }
        }

        public int Add (Customer value) {
            return List.Add(value);
        }

        public Customer AddNew() {
            Console.WriteLine("ADDNEW");	
            return (Customer)((IBindingList)this).AddNew();
        }

        public void Remove (Customer value) {
            List.Remove(value);
        }

        
        protected virtual void OnListChanged(ListChangedEventArgs ev) {
            if (onListChanged != null) {
                 onListChanged(this, ev);
            }
        }
        

        protected override void OnClear() {
            foreach (Customer c in List) {
                c.Parent = null;
            }
        }

        protected override void OnClearComplete() {
            OnListChanged(resetEvent);
        }


        /// <summary>
        ///     IComponent implementation
        /// </summary>
        ISite IComponent.Site { 
            get {
                return component.Site;
            }
            
            set {
                component.Site = value;
            }
        }

        public event EventHandler Disposed;

        void IDisposable.Dispose() {
           component.Dispose();
        }


        /// <summary>
        ///     IBindingList implementation     
        /// </summary>
        bool IBindingList.AllowEdit { 
            get { return true ; }
        }

        bool IBindingList.AllowNew { 
            get { return true ; }
        }

        bool IBindingList.AllowRemove { 
            get { return true ; }
        }

        bool IBindingList.SupportsChangeNotification { 
            get { return true ; }
        }
        
        bool IBindingList.SupportsSearching { 
            get { return false ; }
        }

        bool IBindingList.SupportsSorting { 
            get { return false ; }
        }


        // Events
        public event ListChangedEventHandler ListChanged {
            add {
                onListChanged += value;
            }
            remove {
                onListChanged -= value;
            }
        }

        //Methods
        object IBindingList.AddNew() {
            Console.WriteLine("IBL ADDNEW");	
            Customer c = new Customer(this.Count.ToString());
            List.Add(c);
            return c;
        }


        //Not supported properties
        bool IBindingList.IsSorted { 
            get { throw new NotSupportedException(); }
        }

        ListSortDirection IBindingList.SortDirection { 
            get { throw new NotSupportedException(); }
        }


        PropertyDescriptor IBindingList.SortProperty { 
            get { throw new NotSupportedException(); }
        }


        // Not supported Methods
        void IBindingList.AddIndex(PropertyDescriptor property) {
            throw new NotSupportedException(); 
        }

        void IBindingList.ApplySort(PropertyDescriptor property, ListSortDirection direction) {
            throw new NotSupportedException(); 
        }

        int IBindingList.Find(PropertyDescriptor property, object key) {
            throw new NotSupportedException(); 
        }

        void IBindingList.RemoveIndex(PropertyDescriptor property) {
            throw new NotSupportedException(); 
        }

        void IBindingList.RemoveSort() {
            throw new NotSupportedException(); 
        }

        
        
        
        
        
        
        
        
        
        
        
        protected override void OnInsertComplete(int index, object value) {
            Customer c = (Customer)value;
            c.Parent = this;
            OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, index));
        }

        protected override void OnRemoveComplete(int index, object value) {
            Customer c = (Customer)value;
            c.Parent = this;
            OnListChanged(new ListChangedEventArgs(ListChangedType.ItemDeleted, index));
        }

        protected override void OnSetComplete(int index, object oldValue, object newValue) {
            if (oldValue != newValue) {

                Customer oldcust = (Customer)oldValue;
                Customer newcust = (Customer)newValue;
                
                oldcust.Parent = null;
                newcust.Parent = this;
                
                //Should we fire OnListChanged(new ListChangedEventArgs(ListChangedType.ItemDeleted, index))
                OnListChanged(new ListChangedEventArgs(ListChangedType.ItemAdded, index));
            }
        }
        
        //Called by Customer when it changes
        internal void CustomerChanged(Customer cust) {
            //Need to think of a more efficient way to do this
            //Maybe Customers carry their own index
            int index = List.IndexOf(cust);
            
            OnListChanged(new ListChangedEventArgs(ListChangedType.ItemChanged, index));
        }
        


        //Worker functions to populate the list with data
        private static Customer ReadCustomer1() {
            Customer cust = new Customer("536-45-1245");
            cust.Title = "Mr.";
            cust.FirstName = "Otis";
            cust.LastName = "Redding";
            cust.DateOfBirth = DateTime.Parse("9/9/1941");
            cust.Address = "1 Dock Street,\r\nThe Bay,\r\nGeorgia,\r\nUSA";
            return cust;
        }
        
        private static Customer ReadCustomer2() {
            Customer cust = new Customer("246-12-5645");
            cust.Title = "Mr.";
            cust.FirstName = "James";
            cust.LastName = "Brown";
            cust.DateOfBirth = DateTime.Parse("5/3/1933");
            cust.Address = "45 New Bag Street,\r\nBarnwell,\r\nSouth Carolina,\r\nUSA";
            return cust;
        }
        
        private static Customer ReadCustomer3() {
            Customer cust = new Customer("651-27-8117");
            cust.Title = "Mz.";
            cust.FirstName = "Aretha";
            cust.LastName = "Franklin";
            cust.DateOfBirth = DateTime.Parse("3/25/1942");
            cust.Address = "21 Chain Ave.,\r\nMemphis,\r\nTennessee,\r\nUSA";
            return cust;
        }
        
        private static Customer ReadCustomer4() {
            Customer cust = new Customer("786-34-2114");
            cust.Title = "Mr.";
            cust.FirstName = "Louis";
            cust.LastName = "Armstrong";
            cust.DateOfBirth = DateTime.Parse("8/4/1901");
            cust.Address = "The West End,\r\nNew Orleans,\r\nLousiana,\r\nUSA";
            return cust;
        }
        
        private static Customer ReadCustomer5() {
            Customer cust = new Customer("445-34-4332");
            cust.Title = "Mz.";
            cust.FirstName = "Billie";
            cust.LastName = "Holiday";
            cust.DateOfBirth = DateTime.Parse("4/17/1915");
            cust.Address = "Southern Breeze Ave.,\r\nBaltimore,\r\nMaryland,\r\nUSA";
            return cust;
        }

        public void LoadCustomers() {
            IList l = (IList)this;
            l.Add(ReadCustomer1());
            l.Add(ReadCustomer2());
            l.Add(ReadCustomer3());
            l.Add(ReadCustomer4());
            l.Add(ReadCustomer5());
            OnListChanged(resetEvent);
        }

        public void SaveCustomers() {
        }


    }




    /// <summary>
    ///     Fixes Count Bug
    /// </summary>
    public abstract class CollectionsBase : IList {
        ArrayList list;

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.InnerList"]/*' />
        protected ArrayList InnerList { 
            get { 
                if (list == null)
                    list = new ArrayList();
                return list;
            }
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.List"]/*' />
        protected IList List {
            get { return (IList)this; }
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.Count"]/*' />
        public int Count {
            get {
                Console.WriteLine("Count");
                return ((ICollection)this).Count;
            }
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.Clear"]/*' />
        public void Clear() {
            ((IList)this).Clear();
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.RemoveAt"]/*' />
        public void RemoveAt(int index) {
            ((IList)this).RemoveAt(index);
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.ICollection.Count"]/*' />
        int ICollection.Count {
            // to avoid newing inner list if no items are ever added
            get { return list == null ? 0 : list.Count; }
        }

        bool IList.IsReadOnly {
            get { return InnerList.IsReadOnly; }
        }

        bool IList.IsFixedSize {
            get { return InnerList.IsFixedSize; }
        }

        bool ICollection.IsSynchronized {
            get { return InnerList.IsSynchronized; }
        }

        Object ICollection.SyncRoot {
            get { return InnerList.SyncRoot; }
        }

        void ICollection.CopyTo(Array array, int index) {
            InnerList.CopyTo(array, index);
        }

        Object IList.this[int index] {
            get { 
                if (index < 0 || index >= InnerList.Count)
                    throw new ArgumentOutOfRangeException("ArgumentOutOfRange_Index");
                return InnerList[index]; 
            }
            set { 
                if (index < 0 || index >= InnerList.Count)
                    throw new ArgumentOutOfRangeException("ArgumentOutOfRange_Index");
                OnValidate(value);
                Object temp = InnerList[index];
                OnSet(index, temp, value); 
                InnerList[index] = value; 
                try {
                    OnSetComplete(index, temp, value);
                }
                catch (Exception) {
                    InnerList[index] = temp; 
                    throw;
                }
            }
        }

        bool IList.Contains(Object value) {
            return InnerList.Contains(value);
        }

        int IList.Add(Object value) {
            OnValidate(value);
            OnInsert(InnerList.Count, value);
            int index = InnerList.Add(value);
            try {
                OnInsertComplete(index, value);
            }
            catch (Exception) {
                InnerList.RemoveAt(index);
                throw;
            }
            return index;
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.IList.Clear"]/*' />
        void IList.Clear() {
            OnClear();
            InnerList.Clear();
            OnClearComplete();
        }

        void IList.Remove(Object value) {
            OnValidate(value);
            int index = InnerList.IndexOf(value);
            if (index < 0) throw new ArgumentException("Arg_RemoveArgNotFound");
            OnRemove(index, value);
            InnerList.RemoveAt(index);
            OnRemoveComplete(index, value);
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.IList.RemoveAt"]/*' />
        void IList.RemoveAt(int index) {
            if (index < 0 || index >= InnerList.Count)
                throw new ArgumentOutOfRangeException("ArgumentOutOfRange_Index");
            Object temp = InnerList[index];
            OnRemove(index, temp);
            InnerList.RemoveAt(index);
            OnRemoveComplete(index, temp);
        }

        int IList.IndexOf(Object value) {
            return InnerList.IndexOf(value);
        }

        void IList.Insert(int index, Object value) {
            if (index < 0 || index > InnerList.Count)
                throw new ArgumentOutOfRangeException("ArgumentOutOfRange_Index");
            OnInsert(index, value);
            InnerList.Insert(index, value);
            try {
                OnInsertComplete(index, value);
            }
            catch (Exception) {
                InnerList.RemoveAt(index);
                throw;
            }
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.GetEnumerator"]/*' />
        public IEnumerator GetEnumerator() {
            return InnerList.GetEnumerator();
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.OnSet"]/*' />
        protected virtual void OnSet(int index, Object oldValue, Object newValue) { 
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.OnInsert"]/*' />
        protected virtual void OnInsert(int index, Object value) { 
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.OnClear"]/*' />
        protected virtual void OnClear() { 
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.OnRemove"]/*' />
        protected virtual void OnRemove(int index, Object value) { 
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.OnValidate"]/*' />
        protected virtual void OnValidate(Object value) { 
            if (value == null) throw new ArgumentNullException("value");
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.OnSetComplete"]/*' />
        protected virtual void OnSetComplete(int index, Object oldValue, Object newValue) { 
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.OnInsertComplete"]/*' />
        protected virtual void OnInsertComplete(int index, Object value) { 
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.OnClearComplete"]/*' />
        protected virtual void OnClearComplete() { 
        }

        /// <include file='doc\CollectionBase.uex' path='docs/doc[@for="CollectionBase.OnRemoveComplete"]/*' />
        protected virtual void OnRemoveComplete(int index, Object value) { 
        }

    }


}


